/*
 * Decompression utility for AMI BIOSes.
 *
 * Copyright (C) 2009-2010 coresystems GmbH
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef O_BINARY   // UNIX systems always works with files in binary mode
#define O_BINARY 0 // needed for DOS & Windows
#endif

#if (defined(__APPLE__) || defined(__DJGPP__) || defined(__MINGW32__))
void *memmem(const void *haystack, size_t haystacklen, const void *needle, size_t needlelen);
size_t strnlen(const char *s, size_t maxlen);
char *strndup(const char *s, size_t n);
#endif

// "endian.h" does not exist on (at least) these platforms:
// FreeBSD, NetBSD, OSF/Tru64, HP-UX 10, Solaris, A/UX, Ultrix and
// AIX. It exists on Linux and Irix
#ifdef __linux__
#include <endian.h>
#endif

#if !defined(le32toh) || !defined(le16toh)
#if BYTE_ORDER == LITTLE_ENDIAN
#define le32toh(x) (x)
#define le16toh(x) (x)
#else
#include <byteswap.h>
#define le32toh(x) bswap_32(x)
#define le16toh(x) bswap_16(x)
#endif
#endif

// DJGPP needs to redefine uint32_t for compatability
#ifdef __DJGPP__
#undef uint32_t
#define uint32_t unsigned int
#include <sys/mman.h>
#endif // __DJGPP__

// MINGW needs to define protection bit flags
#ifdef __MINGW32__
#define PROT_NONE	0	// no access to these pages
#define PROT_READ	0x1	// pages can be read
#define PROT_WRITE	0x2	// pages can be written
#define PROT_EXEC	0	// pages can be executed - not used
#endif // __MINGW32__

#if (defined(__DJGPP__) || defined(__MINGW32__))
#include <io.h>
#include <unistd.h>

// DJGPP & MINGW needs to define memory sharing types
#define	MAP_COPY	0x0020	// "copy" region at mmap time
#define	MAP_SHARED	0x0010	// share changes
#define	MAP_PRIVATE	0x0000	// changes are private

// DJGPP & MINGW needs mmap emulation, this one works only for files using buffers in RAM
//#define DEBUG_MMAP
#define MMAP_ENTRIES 1024       // max 1024 regions mapped at one time

// structure for holding info about mapped area
typedef struct
{
  void *addr;
  int prot, fd;
  off_t offset;
} MMAP_ENTRY;

// create file to memory mapping
void *mmap(void *addr, size_t len, int prot, int flags, int fd, off_t offset);

// remove file to memory mapping
int munmap(void *addr, size_t len);

#endif // __DJGPP__, __MINGW32__
